ItemSetGuide.GuiHandler = {}

local libResearchMapping = {
    MainHand = {
        [WEAPONTYPE_AXE] = WEAPONTYPE_AXE,
        [WEAPONTYPE_HAMMER] = WEAPONTYPE_HAMMER,
        [WEAPONTYPE_SWORD] = WEAPONTYPE_SWORD,
        [WEAPONTYPE_TWO_HANDED_AXE] = WEAPONTYPE_TWO_HANDED_AXE,
        [WEAPONTYPE_TWO_HANDED_HAMMER] = WEAPONTYPE_TWO_HANDED_HAMMER,
        [WEAPONTYPE_TWO_HANDED_SWORD] = WEAPONTYPE_TWO_HANDED_SWORD,
        [WEAPONTYPE_DAGGER] = WEAPONTYPE_DAGGER,
        [WEAPONTYPE_BOW] = WEAPONTYPE_BOW,
        [WEAPONTYPE_FIRE_STAFF] = WEAPONTYPE_FIRE_STAFF,
        [WEAPONTYPE_FROST_STAFF] = WEAPONTYPE_FROST_STAFF,
        [WEAPONTYPE_LIGHTNING_STAFF] = WEAPONTYPE_LIGHTNING_STAFF,
        [WEAPONTYPE_HEALING_STAFF] = WEAPONTYPE_HEALING_STAFF,
    },
    OffHand = {
        [WEAPONTYPE_SHIELD] = EQUIP_TYPE_OFF_HAND,
    },
    Head = {
        [ARMORTYPE_LIGHT] = EQUIP_TYPE_HEAD,
        [ARMORTYPE_MEDIUM] = EQUIP_TYPE_HEAD,
        [ARMORTYPE_HEAVY] = EQUIP_TYPE_HEAD,
    },
    Shoulders = {
        [ARMORTYPE_LIGHT] = EQUIP_TYPE_SHOULDERS,
        [ARMORTYPE_MEDIUM] = EQUIP_TYPE_SHOULDERS,
        [ARMORTYPE_HEAVY] = EQUIP_TYPE_SHOULDERS,
    },
    Chest = {
        [ARMORTYPE_LIGHT] = EQUIP_TYPE_CHEST,
        [ARMORTYPE_MEDIUM] = EQUIP_TYPE_CHEST,
        [ARMORTYPE_HEAVY] = EQUIP_TYPE_CHEST,
    },
    Hands = {
        [ARMORTYPE_LIGHT] = EQUIP_TYPE_HAND,
        [ARMORTYPE_MEDIUM] = EQUIP_TYPE_HAND,
        [ARMORTYPE_HEAVY] = EQUIP_TYPE_HAND,
    },
    Waist = {
        [ARMORTYPE_LIGHT] = EQUIP_TYPE_WAIST,
        [ARMORTYPE_MEDIUM] = EQUIP_TYPE_WAIST,
        [ARMORTYPE_HEAVY] = EQUIP_TYPE_WAIST,
    },
    Legs = {
        [ARMORTYPE_LIGHT] = EQUIP_TYPE_LEGS,
        [ARMORTYPE_MEDIUM] = EQUIP_TYPE_LEGS,
        [ARMORTYPE_HEAVY] = EQUIP_TYPE_LEGS,
    },
    Feet = {
        [ARMORTYPE_LIGHT] = EQUIP_TYPE_FEET,
        [ARMORTYPE_MEDIUM] = EQUIP_TYPE_FEET,
        [ARMORTYPE_HEAVY] = EQUIP_TYPE_FEET,
    },
}

function ItemSetGuide.ToggleGui()
    local isHidden = ItemSetGuideMainWindow:IsHidden()
    if isHidden then
        if ItemSetGuideMainWindowContent.currentData == nil then
            ItemSetGuideMainWindowContent.setNewDataFunction(ItemSetGuideMainWindowContent, ItemSetGuide.Data.DataTable)
        end
    end

    ItemSetGuideMainWindow:SetHidden(not isHidden)
end

function ItemSetGuide.ToggleSearchWindow()
    ItemSetGuideMainWindowSearchWindow:SetHidden(not ItemSetGuideMainWindowSearchWindow:IsHidden())
end

function ItemSetGuide.ExecuteSearch()
    local selectedSetType = ItemSetGuideMainWindowSearchWindowSetType.dropdown:GetSelectedItemData()
    local selectedEquipSlot = ItemSetGuideMainWindowSearchWindowSlot.dropdown:GetSelectedItemData()
    local selectedItemType = ItemSetGuideMainWindowSearchWindowItemType.dropdown:GetSelectedItemData()
    local onlyCraftables = ZO_CheckButton_IsChecked(ItemSetGuideMainWindowSearchWindowCraftableOnly)
    local filterTables = {}
    local data = ItemSetGuide.Data.DataTable
    local craftabilityFilter
    local craftabilitySubFilter 

    if selectedItemType ~= nil then
        craftabilitySubFilter = selectedItemType.id
    end

    if selectedSetType.id == ITEMSETGUIDE_SETTYPE_CRAFTED then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.Craftable)        
    elseif selectedSetType.id == ITEMSETGUIDE_SETTYPE_NONCRAFTED then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.NotCraftable)
        --data = ItemSetGuide.Libraries.LibDwemerAssistants.GetTableIntersectionByKey(ItemSetGuide.Data.LookupTables.NotCraftable, ItemSetGuide.Data.DataTable)
    end

    if selectedEquipSlot.id == EQUIP_SLOT_MAIN_HAND then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.MainHandAvailable)
        craftabilityFilter = "MainHand"
    elseif selectedEquipSlot.id == EQUIP_SLOT_OFF_HAND then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.OffHandAvailable)
        craftabilityFilter = "OffHand"
    elseif selectedEquipSlot.id == EQUIP_SLOT_NECK then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.NeckAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_RING1 then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.RingAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_HEAD then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.HeadAvailable)
        craftabilityFilter = "Head"
    elseif selectedEquipSlot.id == EQUIP_SLOT_SHOULDERS then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.ShouldersAvailable)
        craftabilityFilter = "Shoulders"
    elseif selectedEquipSlot.id == EQUIP_SLOT_CHEST then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.ChestAvailable)
        craftabilityFilter = "Chest"
    elseif selectedEquipSlot.id == EQUIP_SLOT_HAND then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.HandsAvailable)
        craftabilityFilter = "Hands"
    elseif selectedEquipSlot.id == EQUIP_SLOT_WAIST then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.WaistAvailable)
        craftabilityFilter = "Waist"
    elseif selectedEquipSlot.id == EQUIP_SLOT_LEGS then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.LegsAvailable)
        craftabilityFilter = "Legs"
    elseif selectedEquipSlot.id == EQUIP_SLOT_FEET then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.FeetAvailable)
        craftabilityFilter = "Feet"
    end

    if selectedEquipSlot.id == EQUIP_SLOT_MAIN_HAND and selectedItemType.id == WEAPONTYPE_AXE then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.AxeAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_MAIN_HAND and selectedItemType.id == WEAPONTYPE_HAMMER then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.HammerAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_MAIN_HAND and selectedItemType.id == WEAPONTYPE_SWORD then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.SwordAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_MAIN_HAND and selectedItemType.id == WEAPONTYPE_BOW then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.BowAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_MAIN_HAND and selectedItemType.id == WEAPONTYPE_HEALING_STAFF then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.HealingStaffAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_MAIN_HAND and selectedItemType.id == WEAPONTYPE_DAGGER then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.DaggerAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_MAIN_HAND and selectedItemType.id == WEAPONTYPE_FIRE_STAFF then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.FireStaffAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_MAIN_HAND and selectedItemType.id == WEAPONTYPE_FROST_STAFF then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.FrostStaffAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_MAIN_HAND and selectedItemType.id == WEAPONTYPE_LIGHTNING_STAFF then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.LightningStaffAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_MAIN_HAND and selectedItemType.id == WEAPONTYPE_TWO_HANDED_AXE then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.TwoHandedAxeAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_MAIN_HAND and selectedItemType.id == WEAPONTYPE_TWO_HANDED_HAMMER then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.TwoHandedHammerAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_MAIN_HAND and selectedItemType.id == WEAPONTYPE_TWO_HANDED_SWORD then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.TwoHandedSwordAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_OFF_HAND and selectedItemType.id == WEAPONTYPE_AXE then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.AxeAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_OFF_HAND and selectedItemType.id == WEAPONTYPE_HAMMER then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.HammerAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_OFF_HAND and selectedItemType.id == WEAPONTYPE_SWORD then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.SwordAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_OFF_HAND and selectedItemType.id == WEAPONTYPE_DAGGER then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.DaggerAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_OFF_HAND and selectedItemType.id == WEAPONTYPE_SHIELD then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.ShieldAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_HEAD and selectedItemType.id == ARMORTYPE_LIGHT then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.HeadLightArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_HEAD and selectedItemType.id == ARMORTYPE_MEDIUM then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.HeadMediumArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_HEAD and selectedItemType.id == ARMORTYPE_HEAVY then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.HeadHeavyArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_SHOULDERS and selectedItemType.id == ARMORTYPE_LIGHT then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.ShouldersLightArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_SHOULDERS and selectedItemType.id == ARMORTYPE_MEDIUM then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.ShouldersMediumArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_SHOULDERS and selectedItemType.id == ARMORTYPE_HEAVY then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.ShouldersHeavyArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_CHEST and selectedItemType.id == ARMORTYPE_LIGHT then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.ChestLightArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_CHEST and selectedItemType.id == ARMORTYPE_MEDIUM then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.ChestMediumArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_CHEST and selectedItemType.id == ARMORTYPE_HEAVY then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.ChestHeavyArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_HAND and selectedItemType.id == ARMORTYPE_LIGHT then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.HandsLightArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_HAND and selectedItemType.id == ARMORTYPE_MEDIUM then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.HandsMediumArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_HAND and selectedItemType.id == ARMORTYPE_HEAVY then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.HandsHeavyArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_WAIST and selectedItemType.id == ARMORTYPE_LIGHT then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.WaistLightArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_WAIST and selectedItemType.id == ARMORTYPE_MEDIUM then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.WaistMediumArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_WAIST and selectedItemType.id == ARMORTYPE_HEAVY then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.WaistHeavyArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_LEGS and selectedItemType.id == ARMORTYPE_LIGHT then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.LegsLightArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_LEGS and selectedItemType.id == ARMORTYPE_MEDIUM then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.LegsMediumArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_LEGS and selectedItemType.id == ARMORTYPE_HEAVY then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.LegsHeavyArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_FEET and selectedItemType.id == ARMORTYPE_LIGHT then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.FeetLightArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_FEET and selectedItemType.id == ARMORTYPE_MEDIUM then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.FeetMediumArmorAvailable)
    elseif selectedEquipSlot.id == EQUIP_SLOT_FEET and selectedItemType.id == ARMORTYPE_HEAVY then
        table.insert(filterTables, ItemSetGuide.Data.LookupTables.FilterTables.FeetHeavyArmorAvailable)
    end

    if #filterTables > 0 then
        for i = 1, #filterTables, 1 do
            data = ItemSetGuide.Libraries.LibDwemerAssistants.GetTableIntersectionByKey(filterTables[i], data)
        end

        if onlyCraftables == true then
            local result = {}

            for key, value in pairs(data) do
                if ItemSetGuide.CheckAnyItemCraftable(value, craftabilityFilter, craftabilitySubFilter) then
                    table.insert(result, value)
                end
            end

            ItemSetGuide.Result = result
            data = result
        else
            data = ItemSetGuide.Libraries.LibDwemerAssistants.ToOrderedTable(data)
        end
    end

    ItemSetGuideMainWindowContent.setNewDataFunction(ItemSetGuideMainWindowContent, data)
end

function ItemSetGuide.CheckAnyItemCraftable(data, craftabilityFilter, craftabilitySubFilter)
    if (craftabilityFilter == nil or craftabilityFilter == "MainHand") and ItemSetGuide.CheckAnyItemInSlotCraftable(data, "MainHand", craftabilitySubFilter) == true then
        return true
    elseif (craftabilityFilter == nil or craftabilityFilter == "OffHand") and ItemSetGuide.CheckAnyItemInSlotCraftable(data, "OffHand", craftabilitySubFilter) == true then
        return true
    elseif (craftabilityFilter == nil or craftabilityFilter == "Head") and ItemSetGuide.CheckAnyItemInSlotCraftable(data, "Head", craftabilitySubFilter) == true then
        return true
    elseif (craftabilityFilter == nil or craftabilityFilter == "Shoulders") and ItemSetGuide.CheckAnyItemInSlotCraftable(data, "Shoulders", craftabilitySubFilter) == true then
        return true
    elseif (craftabilityFilter == nil or craftabilityFilter == "Chest") and ItemSetGuide.CheckAnyItemInSlotCraftable(data, "Chest", craftabilitySubFilter) == true then
        return true
    elseif (craftabilityFilter == nil or craftabilityFilter == "Hands") and ItemSetGuide.CheckAnyItemInSlotCraftable(data, "Hands", craftabilitySubFilter) == true then
        return true
    elseif (craftabilityFilter == nil or craftabilityFilter == "Waist") and ItemSetGuide.CheckAnyItemInSlotCraftable(data, "Waist", craftabilitySubFilter) == true then
        return true
    elseif (craftabilityFilter == nil or craftabilityFilter == "Legs") and ItemSetGuide.CheckAnyItemInSlotCraftable(data, "Legs", craftabilitySubFilter) == true then
        return true
    elseif (craftabilityFilter == nil or craftabilityFilter == "Feet") and ItemSetGuide.CheckAnyItemInSlotCraftable(data, "Feet", craftabilitySubFilter) == true then
        return true
    else
        return false
    end
end

function ItemSetGuide.CheckAnyItemInSlotCraftable(data, itemSlotName, craftabilitySubFilter)
    if data[itemSlotName] == nil then
        return true
    end

    local count = NonContiguousCount(data[itemSlotName])

    for i = 1, count, 1 do
        if craftabilitySubFilter == nil or (data[itemSlotName][i].ArmorType ~= nil and data[itemSlotName][i].ArmorType == craftabilitySubFilter) or (data[itemSlotName][i].WeaponType ~= nil and data[itemSlotName][i].WeaponType == craftabilitySubFilter) then
            local traitCount = ItemSetGuide.CalculateTraitCountForItemSlot(data, itemSlotName, i)

            if traitCount >= data.RequiredTraits then
                return true
            end
        end
    end

    return false
end

function ItemSetGuide.CreateItemLink(item)
    return ZO_LinkHandler_CreateLink(item.LocalizedItemName, nil, ITEM_LINK_TYPE, item.ItemId, ItemSetGuide.Settings.ItemQuality + 1, ItemSetGuide.CalculatedSettings.ItemLevel, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, ITEMSTYLE_RACIAL_WOOD_ELF, 0, 0, 0, 0)
end

function ItemSetGuide.FillDataRow(control)
    local setDataEntry = control.rowData

    local setDataName = ""
    local setBonusText = nil
    local requiredTraits = nil
    local setType = SETTYPE_NONE
    local levels = {}
    local levelsText = nil

    if setDataEntry ~= nil then
        setDataName = setDataEntry.Name
        setBonusText = setDataEntry.SetBonusText
        requiredTraits = setDataEntry.RequiredTraits
        setType = setDataEntry.SetType
        levels = setDataEntry.Levels
        levelsText = setDataEntry.LevelsText
    end

    local setnameControl = control:GetNamedChild("Setname")
    local setnameLabel = setnameControl:GetNamedChild("Label")
    setnameLabel:SetText(setDataName)

    setnameControl.tooltipText = setBonusText

    local craftControl = control:GetNamedChild("Craft")

    if requiredTraits ~= nil then
        craftControl.tooltipText = ItemSetGuide.Localization.RequiredTraitsFormatText:format(requiredTraits)
    else
        craftControl.tooltipText = nil
    end

    local craftLabel = craftControl:GetNamedChild("Label")

    if setType == ITEMSETGUIDE_SETTYPE_CRAFTED then
        craftLabel:SetText(ItemSetGuide.Localization.Yes)
    elseif setType == ITEMSETGUIDE_SETTYPE_NONCRAFTED then
        craftLabel:SetText(ItemSetGuide.Localization.No)
    else
        craftLabel:SetText("")
    end

    local levelControl = control:GetNamedChild("Level")
    local levelLabel = levelControl:GetNamedChild("Label")

    if levels[1] ~= nil then
        levelLabel:SetText(levels[1].Text)
    else
        levelLabel:SetText("")
    end

    levelControl.tooltipText = levelsText

    ItemSetGuide.HandleSetItem(control, setDataEntry, "MH", "MainHand", 1)
    ItemSetGuide.HandleSetItem(control, setDataEntry, "OH", "OffHand", 1)
    ItemSetGuide.HandleSetItem(control, setDataEntry, "Neck", "Neck", 1)
    ItemSetGuide.HandleSetItem(control, setDataEntry, "Ring1", "Ring", 1)
    ItemSetGuide.HandleSetItem(control, setDataEntry, "Ring2", "Ring", 2)
    ItemSetGuide.HandleSetItem(control, setDataEntry, "Head", "Head", 1)
    ItemSetGuide.HandleSetItem(control, setDataEntry, "Shoulders", "Shoulders", 1)
    ItemSetGuide.HandleSetItem(control, setDataEntry, "Chest", "Chest", 1)
    ItemSetGuide.HandleSetItem(control, setDataEntry, "Hands", "Hands", 1)
    ItemSetGuide.HandleSetItem(control, setDataEntry, "Waist", "Waist", 1)
    ItemSetGuide.HandleSetItem(control, setDataEntry, "Legs", "Legs", 1)
    ItemSetGuide.HandleSetItem(control, setDataEntry, "Feet", "Feet", 1)

    ItemSetGuide.HandleLocation(control, setDataEntry, "LocationAldmeriDominion", ALLIANCE_ALDMERI_DOMINION)
    ItemSetGuide.HandleLocation(control, setDataEntry, "LocationEbonheartPact", ALLIANCE_EBONHEART_PACT)
    ItemSetGuide.HandleLocation(control, setDataEntry, "LocationDaggerfallCovenant", ALLIANCE_DAGGERFALL_COVENANT)
end

function ItemSetGuide.HandleLocation(control, value, subControlName, alliance)
    local subControl = control:GetNamedChild(subControlName)    
    local texture = subControl:GetNamedChild("Texture")

    if value == nil or value.Locations == nil then
        texture:SetHidden(true)
        subControl.tooltipText = nil
    else
        texture:SetHidden(false)
        subControl.tooltipText = value.Locations[alliance]
    end
end

function ItemSetGuide.HandleSetItem(control, value, subControlName, itemSlotName, itemIndex)
    local count = 0
    if value ~= nil then
        count = NonContiguousCount(value[itemSlotName])
    end

    local subControl = control:GetNamedChild(subControlName)    

    if count < itemIndex then
        subControl.buttonPlus:SetHidden(true)
        subControl.buttonAccept:SetHidden(true)
        subControl.buttonDecline:SetHidden(true)
        return
    end

    if count >= itemIndex and count > 1 then
        subControl.buttonPlus:SetHidden(false)
        subControl.buttonAccept:SetHidden(true)
        subControl.buttonDecline:SetHidden(true)
    else
        subControl.buttonPlus:SetHidden(true)
        subControl.buttonAccept:SetHidden(false)
        subControl.buttonDecline:SetHidden(true)
    end

    if value.RequiredTraits ~= nil and itemSlotName ~= "Neck" and itemSlotName ~= "Ring" then
        local anyCraftable = false
        local i
        for i = 1, count, 1 do
            local traitCount = ItemSetGuide.CalculateTraitCountForItemSlot(value, itemSlotName, i)

            if traitCount >= value.RequiredTraits then
                anyCraftable = true
                break
            end
        end

        if anyCraftable == false then
            subControl.buttonPlus:SetHidden(true)
            subControl.buttonAccept:SetHidden(true)
            subControl.buttonDecline:SetHidden(false)
        end
    end
end

function ItemSetGuide.CalculateTraitCountForItemSlot(rowData, itemSlotName, i)
    local currentType

    if rowData[itemSlotName][i].WeaponType == nil then
        currentType = rowData[itemSlotName][i].ArmorType
    else
        currentType = rowData[itemSlotName][i].WeaponType
    end

    local traitType = libResearchMapping[itemSlotName][currentType]

    local traitCount = 0

    if rowData[itemSlotName][i].WeaponType == nil then
        traitCount = ItemSetGuide.TraitCounts.Armor[currentType][traitType]
    else
        traitCount = ItemSetGuide.TraitCounts.Weapons[traitType]
    end

    return traitCount
end

function ItemSetGuide.HandleItemButtonMouseEnter(self, rowData, itemSlotName, itemIndex)
    local value = rowData
    if value == nil then
        return
    end

    if value["Has".. itemSlotName] == nil or value[itemSlotName][itemIndex] == nil then
        return
    end

    self.currentTooltip = ItemTooltip

    InitializeTooltip(self.currentTooltip, self, LEFT, 0, 0, 0)
    self.currentTooltip:SetHidden(false)
    self.currentTooltip:ClearLines()

    local item = value[itemSlotName][itemIndex]
    local link = ItemSetGuide.CreateItemLink(item)
    self.currentTooltip:SetLink(link)
end

function ItemSetGuide.HandleItemButtonMouseExit(self)
    if self.currentTooltip == nil then
        return
    end

    ClearTooltip(self.currentTooltip)
    self.currentTooltip:SetHidden(true)
    self.currentTooltip = nil
end

function ItemSetGuide.SetDetailViewData(rowData, itemSlotName)    
    if rowData == nil then
        return
    end

    ItemSetGuideMainWindowDetailView.rowData = rowData
    ItemSetGuideMainWindowDetailView.itemSlotName = itemSlotName

    local i
    for i = 1, 12, 1 do
        if rowData[itemSlotName][i] == nil then
            ItemSetGuideMainWindowDetailView.items[i]:SetHidden(true)
        else
            ItemSetGuideMainWindowDetailView.items[i]:SetHidden(false)

            local icon, sellPrice, meetsUsageRequirement, equipType, itemStyle = GetItemLinkInfo(ItemSetGuide.CreateItemLink(rowData[itemSlotName][i]))
            ItemSetGuideMainWindowDetailView.items[i]:GetNamedChild("Texture"):SetTexture(icon)

            local traitLabel = ItemSetGuideMainWindowDetailView.items[i]:GetNamedChild("Traits")

            if rowData.RequiredTraits ~= nil and itemSlotName ~= "Neck" and itemSlotName ~= "Ring" then
                traitLabel:SetHidden(false)

                local traitCount = ItemSetGuide.CalculateTraitCountForItemSlot(rowData, itemSlotName, i)

                traitLabel:SetText(ItemSetGuide.Localization.TraitFormatText:format(traitCount))

                if traitCount >= rowData.RequiredTraits then
                    traitLabel:SetColor(0, 1, 0, 1)
                else
                    traitLabel:SetColor(1, 0, 0, 1)
                end
            else
                traitLabel:SetHidden(true)
            end
        end
    end
end

function ItemSetGuide.FillDropDown(dropdownControl, choicesTable, order, callback)    
    local emptyItem = dropdownControl.dropdown:CreateItemEntry(" ", callback)
    dropdownControl.dropdown:AddItem(emptyItem, ZO_COMBOBOX_SUPRESS_UPDATE)

    if order == nil then
        order = {}
        for key, _ in pairs(choicesTable) do
            table.insert(order, key)    
        end
    end

    for _, value in pairs(order) do
        if choicesTable[value] ~= nil then
            local item = dropdownControl.dropdown:CreateItemEntry(choicesTable[value], callback)
            item.id = value
            dropdownControl.dropdown:AddItem(item, ZO_COMBOBOX_SUPRESS_UPDATE)
        end
    end

    dropdownControl.dropdown:SelectItem(emptyItem)
end

function ItemSetGuide.EquipSlotSelectionChanged(comboBox, itemName, item)
    ItemSetGuideMainWindowSearchWindowItemType.dropdown:ClearItems()

    if item.id == nil then
        return
    end

    local order = { WEAPONTYPE_AXE, WEAPONTYPE_HAMMER, WEAPONTYPE_SWORD, WEAPONTYPE_DAGGER, WEAPONTYPE_SHIELD, WEAPONTYPE_TWO_HANDED_AXE, WEAPONTYPE_TWO_HANDED_HAMMER, WEAPONTYPE_TWO_HANDED_SWORD, WEAPONTYPE_BOW, WEAPONTYPE_FIRE_STAFF, WEAPONTYPE_FROST_STAFF, WEAPONTYPE_LIGHTNING_STAFF, WEAPONTYPE_HEALING_STAFF, }

    if item.id ~= EQUIP_SLOT_MAIN_HAND and item.id ~= EQUIP_SLOT_OFF_HAND then
        order = { ARMORTYPE_LIGHT, ARMORTYPE_MEDIUM, ARMORTYPE_HEAVY, }
    end

    ItemSetGuide.FillDropDown(ItemSetGuideMainWindowSearchWindowItemType, ItemSetGuide.Localization.ItemTypes[item.id], order)
end

function ItemSetGuide.CraftableOnlyChanged(control, checked)
    if checked == true and ItemSetGuideMainWindowSearchWindowSetType.dropdown:GetSelectedItemData().id ~= ITEMSETGUIDE_SETTYPE_CRAFTED then
        local allItems = ItemSetGuideMainWindowSearchWindowSetType.dropdown:GetItems()
        local selectItem

        for key, value in pairs(allItems) do
            if value.id == ITEMSETGUIDE_SETTYPE_CRAFTED then
                selectItem = value
                break
            end
        end

        ItemSetGuideMainWindowSearchWindowSetType.dropdown:SelectItem(selectItem)
    end
end

function ItemSetGuide.SetTypeSelectionChanged(comboBox, itemName, item)
    if item.id ~= ITEMSETGUIDE_SETTYPE_CRAFTED and ZO_CheckButton_IsChecked(ItemSetGuideMainWindowSearchWindowCraftableOnly) then
        ZO_CheckButton_SetCheckState(ItemSetGuideMainWindowSearchWindowCraftableOnly, false)
    end
end