function ItemSetGuide.ItemDataTracker.AddToTrackedData(id, level, quality, style, trait, itemLink, source)
    if ItemSetGuide.ItemDataTracker.KnownItems[id] == nil then
        return
    end

    if ItemSetGuide.ItemDataTracker.Data.Items[id] == nil then
        ItemSetGuide.ItemDataTracker.Data.Items[id] = {}
        ItemSetGuide.ItemDataTracker.Data.Items[id].Entries = {}
    end

    for key, value in pairs(ItemSetGuide.ItemDataTracker.Data.Items[id].Entries) do
        if value.ItemLink == itemLink then
            value.TimeStamp = GetTimeStamp()
            return
        end
    end

    local entry = {}
    entry.Level = level
    entry.Quality = quality
    entry.Style = style
    entry.Trait = trait
    entry.ItemLink = itemLink
    entry.Source = source
    entry.TimeStamp = GetTimeStamp()

    table.insert(ItemSetGuide.ItemDataTracker.Data.Items[id].Entries, entry)
end

function ItemSetGuide.ItemDataTracker.ProcessItemData(trait, itemLink, source)
    local _, _, _, id, quality, level, enchantmentType, _, _, _, _, _, _, _, _, _, _, _, style, _, _, _ = ZO_LinkHandler_ParseLink(itemLink)

    if id == nil then
        return
    end

    ItemSetGuide.ItemDataTracker.AddToTrackedData(tonumber(id), tonumber(level), tonumber(quality), tonumber(style), tonumber(trait), itemLink, source)
end

function ItemSetGuide.ItemDataTracker.ScanItemSlot(bagId, slotId, source)
    local trait = GetItemTrait(bagId, slotId)
    local itemLink = GetItemLink(bagId, slotId, LINK_STYLE_DEFAULT)    

    ItemSetGuide.ItemDataTracker.ProcessItemData(trait, itemLink, source)
end

function ItemSetGuide.ItemDataTracker.ScanBag(bagId, source)
    local slots = GetBagSize(bagId)

    for i = 1, slots, 1 do
        ItemSetGuide.ItemDataTracker.ScanItemSlot(bagId, i, source)
    end
end

function ItemSetGuide.ItemDataTracker.ScanBank(...)
    ItemSetGuide.ItemDataTracker.ScanBag(BAG_BANK, "Bank")
end

function ItemSetGuide.ItemDataTracker.ScanGuildBank(...)
    local guildId = GetSelectedGuildBankId()
    local guildName = GetGuildName(guildId)    

    local slots = GetBagSize(BAG_GUILDBANK)

    for i = 1, #ZO_GuildBankBackpack.data, 1 do
        local slotId = ZO_GuildBankBackpack.data[i].data.slotIndex

        ItemSetGuide.ItemDataTracker.ScanItemSlot(BAG_GUILDBANK, slotId, "GuildBank "..guildName)
    end
end

function ItemSetGuide.ItemDataTracker.ScanInventory()
    if ItemSetGuide.ItemDataTracker.Data.Settings.InventoriesScanned[GetUnitName('player')] == true then
        return
    end

    ItemSetGuide.ItemDataTracker.ScanBag(BAG_BACKPACK, "Inventory")

    ItemSetGuide.ItemDataTracker.Data.Settings.InventoriesScanned[GetUnitName('player')] = true
end

function ItemSetGuide.ItemDataTracker.InventoryUpdate(eventCode, bagId, slotId, isNewItem, itemSoundCategory, updateReason)
    ItemSetGuide.ItemDataTracker.ScanItemSlot(bagId, slotId, "Inventory")
end

function ItemSetGuide.ItemDataTracker.ScanGuildSalesHistory()
    local guildCount = GetNumGuilds()

    if guildCount == 0 then 
        return 
    end

    for guildId = 1, guildCount, 1 do
        local guildName = GetGuildName(guildId)

        RequestGuildHistoryCategoryNewest(guildId, GUILD_HISTORY_SALES)
        RequestGuildHistoryCategoryOlder(guildId, GUILD_HISTORY_SALES)

        local eventCount = GetNumGuildEvents(guildId, GUILD_HISTORY_SALES)

        for eventIndex = 1, eventCount, 1 do
            local _, _, _, _, _, itemLink, _, _ = GetGuildEventInfo(guildId, GUILD_HISTORY_SALES, eventIndex)

            ItemSetGuide.ItemDataTracker.ProcessItemData(nil, itemLink, "GuildSalesHistory "..guildName)
        end
    end
end

local linkPattern = "|H%d:item:%d-:%d-:%d-:%d-:%d-:%d-:%d-:%d-:%d-:%d-:%d-:%d-:%d-:%d-:%d-:%d-:%d-:%d-:%d-:%d-|h|h"

function ItemSetGuide.ItemDataTracker.ScanChatMessage(eventCode, messageType, fromName, text)
    if text == nil then
        return
    end
    
    local itemLink = string.match(text, linkPattern)
    
    if itemLink ~= nil then
        ItemSetGuide.ItemDataTracker.ProcessItemData(nil, itemLink, "Chat")
    end
end