ItemSetGuide = {
    Name = "ItemSetGuide",
    Version = "1.1",
    SavedVariablesName = "ItemSetGuideSavedVariables",
    Author = "Sephiroth08",
    SlashCommand = "/itemsetguide",
    CalculatedSettings = {
        ItemLevel = 40,
    },
    ResearchMap = {},
    TraitCounts = {
        Weapons = {
        },
        Armor = {
            [ARMORTYPE_LIGHT] = {},
            [ARMORTYPE_MEDIUM] = {},
            [ARMORTYPE_HEAVY] = {},
        },
    },
    Data = {
        DataTable = {},
        LookupTables = {
            FilterTables = {},
            SortingTables = {},
        },
    },
    Libraries = {
        LAM2 = LibStub("LibAddonMenu-2.0"),
        LibSettings = LibStub("LibSettings-1"),
        LibResearch = LibStub("libResearch"),
        LibDwemerAssistants = LibStub("LibDwemerAssistants-1"),
        wm = GetWindowManager(),
    },
    ItemDataTracker = {
        SavedVariablesName = "ItemDataTrackerData",
        Data = {
            Items = {},
            Settings = {
                InventoriesScanned = {},
            },
        },
        DefaultData = {
            Items = {},
            Settings = {
                InventoriesScanned = {},
            },
        },
    }
}

--a variable that takes everything that is only needed on initialization, so that it can be easily cleaned up when the initialization is done
ItemSetGuideInitializer = {}

local optionsPanel 

function ItemSetGuide.Initialize(eventCode, addOnName)
    if (ItemSetGuide.Name ~= addOnName) then
        return
    end

    EVENT_MANAGER:UnregisterForEvent(ItemSetGuide.Name, EVENT_ADD_ON_LOADED)

    ItemSetGuide.Settings = ItemSetGuide.Libraries.LibSettings.LoadAccountWideSettings(ItemSetGuide.SavedVariablesName, 1, ItemSetGuide.Defaults)
    ItemSetGuide.ItemDataTracker.Data = ItemSetGuide.Libraries.LibSettings.LoadAccountWideSettings(ItemSetGuide.ItemDataTracker.SavedVariablesName, 1, ItemSetGuide.ItemDataTracker.DefaultData)

    local settingsPanelIdentifier, settingsPanel, settingsMenu =  ItemSetGuideInitializer.GetSettingsMenuDefinitions()

    optionsPanel = ItemSetGuide.Libraries.LibSettings.CreateSettingsMenu(settingsPanelIdentifier, settingsPanel, settingsMenu, ItemSetGuide.Libraries.LAM2)

    ItemSetGuide.ResearchMap = ItemSetGuide.Libraries.LibResearch:GetResearchMap()

    ItemSetGuideInitializer.InitializeGui()
    ItemSetGuideInitializer = nil

    SLASH_COMMANDS[ItemSetGuide.SlashCommand] = ItemSetGuide.ExecuteSlashCommand

    ZO_CreateStringId("SI_BINDING_NAME_ITEMSETGUIDE_SHOW_GUI", ItemSetGuide.Localization.KeyBindings.ShowGui)

    EVENT_MANAGER:RegisterForEvent(ItemSetGuide.Name, EVENT_PLAYER_ACTIVATED, ItemSetGuide.OnPlayerActivated)
    EVENT_MANAGER:RegisterForEvent(ItemSetGuide.Name, EVENT_LEVEL_UPDATE, ItemSetGuide.CalculateLevel)
    EVENT_MANAGER:RegisterForEvent(ItemSetGuide.Name, EVENT_VETERAN_RANK_UPDATE, ItemSetGuide.CalculateLevel)
    EVENT_MANAGER:RegisterForEvent(ItemSetGuide.Name, EVENT_INVENTORY_SINGLE_SLOT_UPDATE, ItemSetGuide.ItemDataTracker.InventoryUpdate)
    EVENT_MANAGER:RegisterForEvent(ItemSetGuide.Name, EVENT_OPEN_BANK, ItemSetGuide.ItemDataTracker.ScanBank)
    EVENT_MANAGER:RegisterForEvent(ItemSetGuide.Name, EVENT_GUILD_BANK_ITEMS_READY, ItemSetGuide.ItemDataTracker.ScanGuildBank)
    --Use this and parse the chat for item links. Condition |H\d:item:\.*|h
    EVENT_MANAGER:RegisterForEvent(ItemSetGuide.Name, EVENT_CHAT_MESSAGE_CHANNEL, ItemSetGuide.ItemDataTracker.ScanChatMessage) 
    --API3 required for using the following events
    --EVENT_MANAGER:RegisterForEvent(addonName,EVENT_LOOT_RECEIVED,OnReceiveLoot)
    --EVENT_LOOT_RECEIVED (integer eventCode, string receivedBy, string itemName, integer quantity, integer itemSound, integer lootType, bool self) 
    --EVENT_TRADING_HOUSE_SEARCH_RESULTS_RECEIVED (integer eventCode, integer guildId, integer numItemsOnPage, integer currentPage, bool hasMorePages) 
    --EVENT_LOOT_ITEM_FAILED (integer eventCode, integer reason, string itemName) 
end

function ItemSetGuide.OnPlayerActivated()
    EVENT_MANAGER:UnregisterForEvent(ItemSetGuide.Name, EVENT_PLAYER_ACTIVATED)

    ItemSetGuide.CalculatedSettings.ItemLevel = GetUnitLevel("player")
    ItemSetGuide.CalculateResearchTraits()

    if ItemSetGuide.Settings.Debug == true then
        ItemSetGuide.ToggleGui()
    end

    ItemSetGuide.ItemDataTracker.ScanInventory()
    ItemSetGuide.ItemDataTracker.ScanGuildSalesHistory()
    
    ItemSetGuide.ReplaceItemSetInformations()
end

function ItemSetGuide.ReplaceItemSetInformations()
    for key, value in pairs(ItemSetGuide.Data.DataTable) do
        local link
        link = ItemSetGuide.ReplaceItemInformations(value, "MainHand", link)
        link = ItemSetGuide.ReplaceItemInformations(value, "OffHand", link)
        link = ItemSetGuide.ReplaceItemInformations(value, "Neck", link)
        link = ItemSetGuide.ReplaceItemInformations(value, "Ring", link)
        link = ItemSetGuide.ReplaceItemInformations(value, "Head", link)
        link = ItemSetGuide.ReplaceItemInformations(value, "Shoulders", link)
        link = ItemSetGuide.ReplaceItemInformations(value, "Chest", link)
        link = ItemSetGuide.ReplaceItemInformations(value, "Hands", link)
        link = ItemSetGuide.ReplaceItemInformations(value, "Waist", link)
        link = ItemSetGuide.ReplaceItemInformations(value, "Legs", link)
        link = ItemSetGuide.ReplaceItemInformations(value, "Feet", link)
        
        local _, setName, numBonuses, _, _ = GetItemLinkSetInfo(link)
        
        value.LocalizedName = setName
        
        value.Bonuses = {}
        value.SetBonusText = ""
        
        for i = 1, numBonuses, 1 do
            local numRequired, bonusDescription = GetItemLinkSetBonusInfo(link, false, i)
            
            table.insert(value.Bonuses, { RequiredItems = numRequired, Description = bonusDescription })
            
            if value.SetBonusText ~= "" then
                value.SetBonusText = value.SetBonusText.."\n"
            end
            
            value.SetBonusText = value.SetBonusText..bonusDescription
        end
    end
end

function ItemSetGuide.ReplaceItemInformations(set, slot, lastLink)
    if set["Has"..slot] ~= true then
        return lastLink
    end
    
    local link
    for key, value in pairs(set[slot]) do
        link = ItemSetGuide.CreateItemLink(value)
        
        local itemName = GetItemLinkName(link)
        
        value.LocalizedItemName = itemName
    end
    
    return link
end

function ItemSetGuide.ExecuteSlashCommand(params)
    local lowerParams = string.lower(params)

    if lowerParams == "showmenu" then
        ItemSetGuide.Libraries.LAM2:OpenToPanel(optionsPanel)
    elseif lowerParams == "show" then
        ItemSetGuide.ToggleGui()
    else
        CHAT_SYSTEM:AddMessage("========================================")
        CHAT_SYSTEM:AddMessage(ItemSetGuide.Name.." Version "..ItemSetGuide.Version.." by "..ItemSetGuide.Author)
        CHAT_SYSTEM:AddMessage("========================================")
        CHAT_SYSTEM:AddMessage(ItemSetGuide.Localization.KeyBindings.CommandsText)
        CHAT_SYSTEM:AddMessage(ItemSetGuide.SlashCommand.." show")
        CHAT_SYSTEM:AddMessage(ItemSetGuide.Localization.KeyBindings.ShowDescription)
        CHAT_SYSTEM:AddMessage("________________________________________")
        CHAT_SYSTEM:AddMessage(ItemSetGuide.SlashCommand.." showmenu")
        CHAT_SYSTEM:AddMessage(ItemSetGuide.Localization.KeyBindings.ShowMenuDescription)
        CHAT_SYSTEM:AddMessage("========================================")        
    end
end

function ItemSetGuide.CalculateResearchTraits()
    for key, value in pairs(ItemSetGuide.ResearchMap) do
        for key2, value2 in pairs(value) do
            for key3, value3 in pairs(value2) do
                if key == CRAFTING_TYPE_CLOTHIER then
                    ItemSetGuide.CalculateResearchTraitLine(key, key2, key3, value3, 0)
                    ItemSetGuide.CalculateResearchTraitLine(key, key2, key3, value3, 7)
                else
                    ItemSetGuide.CalculateResearchTraitLine(key, key2, key3, value3, 0)
                end                
            end
        end
    end
end

function ItemSetGuide.CalculateResearchTraitLine(key, key2, key3, value3, addToValue)
    local armorSlot
    if key2 == "ARMOR" and key3 ~= EQUIP_TYPE_OFF_HAND then
        if key == CRAFTING_TYPE_BLACKSMITHING then
            armorSlot = ARMORTYPE_HEAVY
        elseif key == CRAFTING_TYPE_CLOTHIER and addToValue == 0 then
            armorSlot = ARMORTYPE_LIGHT
        elseif key == CRAFTING_TYPE_CLOTHIER and addToValue == 7 then
            armorSlot = ARMORTYPE_MEDIUM
        end
    end

    if ItemSetGuide.TraitCounts.Weapons[key3] == nil or ItemSetGuide.TraitCounts.Armor[armorSlot][key3] == nil then
        if key2 == "WEAPON" or key3 == EQUIP_TYPE_OFF_HAND then
            ItemSetGuide.TraitCounts.Weapons[key3] = 0
        else
            ItemSetGuide.TraitCounts.Armor[armorSlot][key3] = 0
        end
    end

    local currentCount = 0

    for i = 1, 8, 1 do
        local _, _, known = GetSmithingResearchLineTraitInfo(key, value3 + addToValue, i)

        if known == true then
            currentCount = currentCount + 1
        end
    end

    if (key2 == "WEAPON" or key3 == EQUIP_TYPE_OFF_HAND) and ItemSetGuide.TraitCounts.Weapons[key3] < currentCount then
        ItemSetGuide.TraitCounts.Weapons[key3] = currentCount
    elseif key2 == "ARMOR" and key3 ~= EQUIP_TYPE_OFF_HAND and ItemSetGuide.TraitCounts.Armor[armorSlot][key3] < currentCount then
        ItemSetGuide.TraitCounts.Armor[armorSlot][key3] = currentCount
    end
end

function ItemSetGuide.DebugOutput(message)
    if ItemSetGuide.Settings.Debug then
        d(message)
    end
end

EVENT_MANAGER:RegisterForEvent(ItemSetGuide.Name, EVENT_ADD_ON_LOADED, ItemSetGuide.Initialize)