local majorVersion = "LibDwemerAssistants-1"
local minorVersion = "0.1"

local libDwemerAssistants = LibStub:NewLibrary(majorVersion, minorVersion)

if libDwemerAssistants == nil then
    return    -- already loaded and no upgrade necessary
end

function libDwemerAssistants.MergeTables(table1, table2)
    local result = {}

    result = ZO_ShallowTableCopy(table1, result)

    for k,v in pairs(table2) do 
        result[k] = v 
    end

    return result
end

function libDwemerAssistants.GetTableIntersection(a, b)
    local result = {}

    for key, value in ipairs(b) do
        if libDwemerAssistants.FindInTable(a, value) then 
            table.insert(result, value)
        end
    end

    return result
end

function libDwemerAssistants.FindInTable(table, element)
    for _, value in pairs(table) do
        if value == element then
            return true
        end
    end
    return false
end

function libDwemerAssistants.GetTableIntersectionByKey(sourceTable, joinTable)
    local result = {}

    for key, value in pairs(joinTable) do
        if sourceTable[key] ~= nil then
            result[key] = value
        end
    end

    return result
end

function libDwemerAssistants.ToOrderedTable(sourceTable)
    local result = {}
    local numEntries = NonContiguousCount(sourceTable)
    local currentEntry = 1
    local currentIndex = 1

    while currentEntry <= numEntries do
        if sourceTable[currentIndex] ~= nil then
            table.insert(result, sourceTable[currentIndex])
            currentEntry = currentEntry + 1
        end
        currentIndex = currentIndex + 1
    end

    return result
end

function libDwemerAssistants.CreateScrollControl(wm, container, headerRow, createRowFunction, rowDataUpdateFunction, initialData)
    if headerRow ~= nil then
        headerRow:ClearAnchors()
        headerRow:SetAnchor(TOPLEFT, container, TOPLEFT, 3, 0)
    end

    container.rows = {}

    libDwemerAssistants.CreateScrollContainerRow(container, headerRow, 1, createRowFunction, rowDataUpdateFunction, initialData)

    local availableHeight = container:GetHeight()
    if headerRow ~= nil then
        availableHeight = availableHeight - headerRow:GetHeight()
    end

    local rowHeight = container.rows[1]:GetHeight()

    container.numberOfRows = math.floor(availableHeight / rowHeight)
    container.setNewDataFunction = function(control, data) libDwemerAssistants.SetNewDataForScrollContainer(control, data) end
    container.currentData = initialData
    container.currentDataRow = 1
    container:SetMouseEnabled(true)
    container:SetHandler("OnMouseWheel",function(self,delta)
            local targetRow = math.max(1, container.currentDataRow - delta)
            local rowCount = NonContiguousCount(container.currentData)

            if targetRow > (rowCount - container.numberOfRows + 1) then
                targetRow = math.max(1, rowCount - container.numberOfRows + 1)
            elseif targetRow < 1 then
                targetRow = 1
            end

            container.slider:SetValue(targetRow)
            container.currentDataRow = targetRow

            libDwemerAssistants.UpdateDataForScrollContainer(container)
        end)

    local slider = wm:CreateControl(container:GetName().."Slider", container, CT_SLIDER)
    slider:SetDimensions(22, availableHeight - 16 - 16 - 16)

    if headerRow ~= nil then
        slider:SetAnchor(TOPRIGHT, container, TOPRIGHT, 0, headerRow:GetHeight() + 16)
    else
        slider:SetAnchor(TOPRIGHT, container, TOPRIGHT, 0, 16)
    end
    slider:SetMouseEnabled(true)
    slider:SetMinMax(1, 100)
    slider:SetValue(1)
    slider:SetValueStep(1)
    slider:SetBackgroundMiddleTexture("EsoUI/Art/ChatWindow/chat_scrollbar_track.dds")
    slider:SetThumbTexture("EsoUI/Art/ChatWindow/chat_thumb.dds", "EsoUI/Art/ChatWindow/chat_thumb_disabled.dds", "EsoUI/Art/ChatWindow/chat_thumb.dds", 8, 50, 0, 0, 0.6875)
    slider:SetHandler("OnValueChanged",function(self,value,eventReason)
            container.currentDataRow = math.max(1, value)
            libDwemerAssistants.UpdateDataForScrollContainer(container)
        end)

    container.slider = slider

    local sliderUp = wm:CreateControlFromVirtual(container:GetName().."Slider".."Up", container, "ZO_ScrollUpButton")
    sliderUp:SetAnchor(BOTTOM, slider, TOP, -1)
    sliderUp:SetNormalTexture("EsoUI/Art/ChatWindow/chat_scrollbar_upArrow_up.dds")
    sliderUp:SetPressedTexture("EsoUI/Art/ChatWindow/chat_scrollbar_upArrow_down.dds")
    sliderUp:SetMouseOverTexture("EsoUI/Art/ChatWindow/chat_scrollbar_upArrow_over.dds")
    sliderUp:SetDisabledTexture("EsoUI/Art/ChatWindow/chat_scrollbar_upArrow_disabled.dds")
    sliderUp:SetHandler("OnMouseDown", function(self, button, ...)
            container.slider:SetValue(math.max(1, container.currentDataRow - 1))
        end)

    local sliderDown = wm:CreateControlFromVirtual(container:GetName().."Slider".."Down", container, "ZO_ScrollDownButton")
    sliderDown:SetAnchor(TOP, slider, BOTTOM, -1)
    sliderDown:SetNormalTexture("EsoUI/Art/ChatWindow/chat_scrollbar_downArrow_up.dds")
    sliderDown:SetPressedTexture("EsoUI/Art/ChatWindow/chat_scrollbar_downArrow_down.dds")
    sliderDown:SetMouseOverTexture("EsoUI/Art/ChatWindow/chat_scrollbar_downArrow_over.dds")
    sliderDown:SetDisabledTexture("EsoUI/Art/ChatWindow/chat_scrollbar_downArrow_disabled.dds")
    sliderDown:SetHandler("OnMouseDown", function(self, button, ...)
            container.slider:SetValue(math.max(1, container.currentDataRow + 1))
        end)  

    local sliderEnd = wm:CreateControlFromVirtual(container:GetName().."Slider".."End", container, "ZO_ScrollDownButton")
    sliderEnd:SetAnchor(TOP, sliderDown, BOTTOM, 0.5)
    sliderEnd:SetNormalTexture("EsoUI/Art/ChatWindow/chat_scrollbar_endArrow_up.dds")
    sliderEnd:SetPressedTexture("EsoUI/Art/ChatWindow/chat_scrollbar_endArrow_down.dds")
    sliderEnd:SetMouseOverTexture("EsoUI/Art/ChatWindow/chat_scrollbar_endArrow_over.dds")
    sliderEnd:SetDisabledTexture("EsoUI/Art/ChatWindow/chat_scrollbar_endArrow_disabled.dds")
    sliderEnd:SetHandler("OnMouseDown", function(self, button, ...)
            local min, max = container.slider:GetMinMax()
            container.slider:SetValue(max)
        end)

    local i

    for i = 2, container.numberOfRows, 1 do
        libDwemerAssistants.CreateScrollContainerRow(container, headerRow, i, createRowFunction, rowDataUpdateFunction, initialData)
    end

    if initialData ~= nil then
        libDwemerAssistants.SetNewDataForScrollContainer(container, initialData)
    end
end

function libDwemerAssistants.CreateScrollContainerRow(container, headerRow, index, createRowFunction, rowDataUpdateFunction, initialData)
    container.rows[index] = createRowFunction(container, index)

    if index == 1 then
        if headerRow ~= nil then
            container.rows[index]:SetAnchor(TOPLEFT, headerRow, BOTTOMLEFT)
        else
            container.rows[index]:SetAnchor(TOPLEFT, container, TOPLEFT)
        end
    else
        container.rows[index]:SetAnchor(TOPLEFT, container.rows[index - 1], BOTTOMLEFT, 0, 0)
    end

    container.rows[index].rowDataUpdateFunction = rowDataUpdateFunction

    if initialData ~= nil and initialData[index] ~= nil then
        container.rows[index].rowData = initialData[index]
        container.rows[index].rowDataUpdateFunction(container.rows[index])
    end
end

function libDwemerAssistants.SetNewDataForScrollContainer(control, data)
    control.currentData = data

    control.slider:SetValue(1)
    local rowCount = NonContiguousCount(data)
    control.slider:SetMinMax(1, math.max(1, rowCount - control.numberOfRows + 1))
    control.currentDataRow = 1

    libDwemerAssistants.UpdateDataForScrollContainer(control)
end

function libDwemerAssistants.UpdateDataForScrollContainer(control)
    local i

    for i = 1, control.numberOfRows, 1 do
        control.rows[i].rowData = control.currentData[control.currentDataRow + i - 1]
        if control.rows[i].rowData == nil then
            control.rows[i]:SetHidden(true)
        else
            control.rows[i]:SetHidden(false)
        end

        control.rows[i].rowDataUpdateFunction(control.rows[i])
    end
end