ItemSetGuide.Settings = {
    Debug = false,
    ItemQuality = 2,
    ItemLevelCalculation = 2,
    FixedLevel = 50,
}
ItemSetGuide.Defaults = {
    Debug = false,
    ItemQuality = 2,
    ItemLevelCalculation = 2,
    FixedLevel = 50,
}

local levelChoices = {}
local reverseLevelChoices = {}
for key, value in pairs(ItemSetGuide.Localization.LevelChoices.Values) do
    levelChoices[key] = value
    reverseLevelChoices[value] = key
end

local qualityChoices = {}
local reverseQualityChoices = {}
for i = 0, 5 do
    local color = GetItemQualityColor(i)
    local qualName = color:Colorize(GetString("SI_ITEMQUALITY", i))
    qualityChoices[i] = qualName
    reverseQualityChoices[qualName] = i
end

function ItemSetGuideInitializer.GetSettingsMenuDefinitions()
    local settingsPanelIdentifier = "ItemSetGuidePanel"
    local settingsPanel = {
        type = "panel",
        name = ItemSetGuide.Name,
        author = ItemSetGuide.Author,
        version = ItemSetGuide.Version,
        registerForRefresh = true, --boolean (optional) (will refresh all options controls when a setting is changed and when the panel is shown)
        registerForDefaults = true, --boolean (optional) (will set all options controls back to default values)
    }
    local settingsMenu = {
        --        [1] = {
        --            type = "header",
        --            name = ItemSetGuide.Localization.DebugHeader,
        --            width = "full"
        --        },
        [1] = {
            type = "dropdown",
            name = ItemSetGuide.Localization.QualityDropdown.Text,
            tooltip = ItemSetGuide.Localization.QualityDropdown.Tooltip,
            choices = qualityChoices,
            getFunc = function() return qualityChoices[ItemSetGuide.Settings.ItemQuality] end,
            setFunc = function(choice) ItemSetGuide.Settings.ItemQuality = reverseQualityChoices[choice] end,
            default = qualityChoices[ItemSetGuide.Defaults.ItemQuality],
        },
        [2] = {
            type = "dropdown",
            name = ItemSetGuide.Localization.LevelChoices.Text,
            tooltip = ItemSetGuide.Localization.LevelChoices.Tooltip,
            choices = levelChoices,
            getFunc = function() return levelChoices[ItemSetGuide.Settings.ItemLevelCalculation] end,
            setFunc = function(choice) 
                ItemSetGuide.Settings.ItemLevelCalculation = reverseLevelChoices[choice] 
                ItemSetGuide.CalculateLevel()
            end,
            default = qualityChoices[ItemSetGuide.Defaults.ItemLevelCalculation],
        },
        [3] = {
            type = "slider",
            name = ItemSetGuide.Localization.FixedLevel.Text,
            tooltip = ItemSetGuide.Localization.FixedLevel.Tooltip,
            min = 1,
            max = 50,
            step = 1,
            getFunc = function() return ItemSetGuide.Settings.FixedLevel end,
            setFunc = function(value) 
                ItemSetGuide.Settings.FixedLevel = value 
                ItemSetGuide.CalculateLevel()            
            end,
            disabled = function() 
                if ItemSetGuide.Settings.ItemLevelCalculation ~= 1 then 
                    return true 
                else 
                    return false 
                end 
            end,
            default = ItemSetGuide.Defaults.FixedLevel,
        },
        --        [3] = {
        --            type = "checkbox",
        --            name = ItemSetGuide.Localization.DebugCheckbox.Text,
        --            tooltip = ItemSetGuide.Localization.DebugCheckbox.Tooltip,
        --            getFunc = function() return ItemSetGuide.Settings.Debug end,
        --            setFunc = function(value) ItemSetGuide.Settings.Debug = value end,
        --            width = "full",
        --            default = ItemSetGuide.Defaults.Debug
        --        },
        [4] = {
            type = "button",
            name = "Open Window",
            tooltip = "Open Window",
            func = function() ItemSetGuide.ToggleGui() end
        },
    }

    return settingsPanelIdentifier, settingsPanel, settingsMenu
end

function ItemSetGuide.CalculateLevel(...)
    if ItemSetGuide.Settings.ItemLevelCalculation == 2 then
        ItemSetGuide.CalculatedSettings.ItemLevel = GetUnitLevel('player')
    elseif ItemSetGuide.Settings.ItemLevelCalculation == 1 then
        ItemSetGuide.CalculatedSettings.ItemLevel = ItemSetGuide.Settings.FixedLevel
    end
end